import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

class TableTile<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const TableTile({
    required this.index,
    required this.viewModel,
    required this.child,
  });

  final int index;
  final Widget child;
  final EntityTableViewModel<T> viewModel;

  @override
  State<TableTile> createState() {
    return _TableTileState<T>();
  }
}

class _TableTileState<T> extends State<TableTile<T>> {
  late bool isSelectorOn = widget.viewModel.state.isSelectorOn;
  late bool isSelected = getIsSelected();

  @override
  void initState() {
    super.initState();
    subscribeToEvents();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  void subscribeToEvents() {
    widget.viewModel.state.subscribe(onStateChanged);
  }

  void unsubscribeToEvents() {
    widget.viewModel.state.unsubscribe(onStateChanged);
  }

  void onStateChanged(TableState state) {
    setState(() {
      isSelectorOn = state.isSelectorOn;
    });
  }

  bool getIsSelected() {
    return widget.viewModel.state.selectedTiles.where((element) => element == widget.index).isNotEmpty;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      key: UniqueKey(),
      padding: const EdgeInsets.symmetric(horizontal: 5),
      child: Row(
        children: [
          isSelectorOn
              ? Checkbox(
                  value: isSelected,
                  activeColor: widget.viewModel.appViewModel.colors.tertiary,
                  side: BorderSide(color: widget.viewModel.appViewModel.colors.onBackground),
                  onChanged: (value) {
                    setState(() {
                      isSelected = !isSelected;
                      if (isSelected) {
                        widget.viewModel.selectTile(widget.index);
                      } else {
                        widget.viewModel.unselectTile(widget.index);
                      }
                    });
                  },
                )
              : Container(),
          Expanded(child: widget.child),
        ],
      ),
    );
  }
}

class TableTileTemplate<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const TableTileTemplate({
    required this.data,
  });

  final T data;

  @override
  State<TableTileTemplate> createState() {
    // ignore: no_logic_in_create_state
    return _TableTileTemplateState<T>(
      data: data,
    );
  }
}

class _TableTileTemplateState<T> extends State<TableTileTemplate> {
  _TableTileTemplateState({
    required this.data,
  });

  final T data;

  @override
  Widget build(BuildContext context) {
    return Container();
  }
}
